/************************************************************
 *
 *	Created: Saturday, May 11, 1991 12:55:02 PM
 *	NeoTypes.h
 *	Type definitions NeoAccess database system
 *
 *
 *	Copyright  Neologic Systems 1992-1994. All Rights Reserved.
 *	All rights reserved
 *
 *
 ***********************************************************/
#ifdef _MSC_VER
#pragma warning( disable :  4004 4068 4080 4355)
#endif

#pragma once			/* Include this file only once */
#ifndef __NeoTypes__
#define __NeoTypes__ 1

#include <stdArg.h>

// This symbol should ALWAYS be defined during development. It can save you hours of debugging!
// #define qNeoDebug 1

#define qNeoPersist
// #define qNeoPersistPlusPlus

#define qNeoVersions 1

// Define this symbol in order to maintain file format compatability with version 2.0 -> 2.1.
// #define qNeo2P0FileFormat

typedef long NeoLockType;
const NeoLockType kNeoDefaultLock		= -1;
const NeoLockType kNeoNoLock			= 0;
const NeoLockType kNeoSnapshotLock		= 1;
const NeoLockType kNeoMonitorLock		= 2;
const NeoLockType kNeoSharedLock		= 3;
const NeoLockType kNeoExclusiveLock		= 4;

typedef unsigned short NeoDirty;
const NeoDirty kNeoClean				= 0;
const NeoDirty kNeoAdded				= 1;
const NeoDirty kNeoChanged				= 2;
const NeoDirty kNeoRemoved				= 3;
const NeoDirty kNeoPeerDirtied			= 4;
const NeoDirty kNeoLockChanged			= 8;

#define NeoMax(a, b)			(((a) > (b)) ? (a) : (b))
#define NeoMin(a, b)			(((a) < (b)) ? (a) : (b))

typedef unsigned short			NeoRefCnt;
const NeoRefCnt	kNeoRefCntMax	= 255;

typedef unsigned long			NeoMark;
typedef unsigned long			NeoID;
typedef void *					NeoPtr;
typedef void *					NeoVoidStar;

typedef enum {
	kNeo2P0FileFormat		= 0x0200,
	kNeo2P1FileFormat		= 0x0201,
	kNeo3P0FileFormat		= 0x0300,
	kNeoCurrentFileFormat	= kNeo3P0FileFormat
} NeoFormat;

#ifdef qNeoVersions
typedef unsigned long			NeoVersion;
const NeoVersion kNeoVersionIncrement	= 0xFFFFFFFF;
const NeoVersion kNeoVersionTime 		= 0xFFFFFFFE;
const NeoVersion kNeoVersionKeep 		= 0;
const NeoVersion kNeoVersionDefault		= 1;
#endif

typedef enum {
	kNeoSimpleRef,
	kNeoReadRef,
	kNeoWriteRef
} NeoRefType;

typedef unsigned long			NeoCheckpoint;

typedef unsigned long			NeoTag;
typedef unsigned long			NeoSelectType;
const NeoSelectType kNeoNullSelect	= 0;

// The primary index of a class is always the first in the class and metaclass
enum {kNeoPrimaryIndex			= 0};
enum {kNeoSecondaryIndex		= 1};
enum {kNeoTeraciaryIndex		= 2};
enum {kNeoQuadaryIndex			= 3};

typedef long NeoOrder;
const NeoOrder kNeoNone			= -1;
const NeoOrder kNeoNear			= 0;
const NeoOrder kNeoLow			= 1;
const NeoOrder kNeoExact		= 2;
const NeoOrder kNeoLowOrEqual	= 3;
const NeoOrder kNeoHigh			= 4;
const NeoOrder kNeoHighOrEqual	= 6;
const NeoOrder kNeoAll			= 7;

#include "NeoMacintosh.h"

const NeoTag kNeoNoTag			= Neo4CharConst('nu', 'll');
const NeoTag kNeoObjectTag		= 1;
const NeoTag kNeoAllTag			= Neo4CharConst('al', 'l ');
const NeoTag kNeoEveryTag		= 0xFFFFFFFF;

typedef unsigned long NeoKeyOp;
const NeoKeyOp kNeoCanSupport		= Neo4CharConst('nc', 'sp');
const NeoKeyOp kNeoGetKey			= Neo4CharConst('ng', 'ky');

class							CNeoStream;
class							CNeoSelect;
class							CNeoDatabase;
class							CNeoPersist;
class							CNeoNode;
class							CNeoClass;

enum {kNeoReferTableSize		= 64};
typedef CNeoPersist *	NeoReferTable[kNeoReferTableSize];
enum {kNeoBusyTableSize			= 32};
typedef CNeoPersist *	NeoBusyTable[kNeoBusyTableSize];

extern CNeoDatabase *			gNeoDatabase;	// The database of the current document is stored in this global
#define gNeoFile				gNeoDatabase	// For compatability with pre-NeoAccess 3.0

#define NeoStillNeed(t)			((CNeoPersist::FCacheUsed <= CNeoPersist::FPurgeStart - (t)) ? (t) = 0 : (t))

#define NeoUsed(x)				(void)(x)

// The previous _new_handler address.
extern NeoNewHandler			gNeoOldNewHandler;

typedef void *					(*NeoTestFunc1)(CNeoNode *aNode, const short aIndex, const NeoLockType aLock, void *aParam);
typedef void *					(*NeoTestClassFunc)(CNeoClass *aNode, const short aIndex, void *aParam);

typedef struct {
	CNeoSelect *				fKey;
	CNeoNode *					fNode;
	short						fIndex;
} NeoFindParam;							// Used when looking for an index with objects that match the key

#ifndef TRUE
#define TRUE					1
#endif
#ifndef FALSE
#define FALSE					0
#endif

#define Neostrncmp(s1, s2, n)	strncmp((char *)(s1), (char *)(s2), (n))
#define Neostrncpy(s1, s2, n)	strncpy((s1), (s2), (n))

#define NeoCacheMalloc(s)		CNeoPersist::CacheMalloc((size_t)(s))
#define NeoCacheFree(o, s)		(CNeoPersist::FCacheUsed -= (s), NeoFree(o))
#define NeoBlobCacheMalloc(s)	CNeoBlob::CacheMalloc((size_t)(s))
#define NeoBlobCacheFree(o, s)	(CNeoPersist::FCacheUsed -= (s), NeoBlobFree(o))

// Macros for making debugging assertions
#ifdef qNeoDebug
#define NeoVerifyNode(node)		(node)->verify(nil)
#else
#define NeoVerifyNode(node)
#endif
#define NeoFailNil(x)			FailNIL((void *)(x))

const NeoID kNeoNullClassID		= 0;
const NeoID kNeoPersistID		= 1;
const NeoID kNeoBlobID			= 2;
const NeoID kNeoClassID			= 3;
const NeoID kNeoSubclassID		= 4;
const NeoID kNeoFreeListID		= 5;
const NeoID kNeoInodeID			= 6;
const NeoID kNeoIDIndexID		= 7;
const NeoID kNeoParentIndexID	= 8;
const NeoID kNeoGenericIndexID	= 9;
const NeoID kNeoPartMgrID		= 10;
const NeoID kNeoIDListID		= 11;

enum {kNeoMaxSuper				= 1};	// Total number of superclasses a class can have
enum {kNeoMaxIndice				= 1};	// Total number of indexes that a class can have.

const NeoMark	kNeoMinMark		= 0x0;

const NeoID	kNeoNoDocumentID	= 0xFFFFFFFF;
const NeoID	kNeoNoPeerID		= ((NeoID)0xFFFFFFFF);

#define	kNeoNoTransactionID		0

#define CNeoAppRootH			CNeoAppNativeH
#define CNeoAppRoot				CNeoAppNative

#define CNeoDocRootH			CNeoDocNativeH
#define CNeoDocRoot				CNeoDocNative

#define CNeoDatabaseRootH		CNeoDatabaseNativeH
#define CNeoDatabaseRoot		CNeoDatabaseNative
#endif
